-- Paul Caspi and Paolo Torrini, Verimag, 2007

module Algo1 where

import Prelude
import List

-- Event Tree structure : this is the input of the analysis

data EventTree  = NilEv	|	      -- degenerated case
		  EC Event [SimpleCond]   -- the event is fired when 
 deriving (Ord, Eq, Show)                 -- the conditions are true 

data Event  = Cont String  | 	-- an event is either controllable
	      NCont String |	-- or non controllable
              Rare String
 deriving (Ord, Eq, Show)                

-- data CondTree = NilCond |		 -- degenerated case
--  		Cand SimpleCond CondTree -- conjunction of conditions

data SimpleCond = COr Cond [SimpleCond] |
                  CE Cond EventTree -- the condition becomes true
				    --  as soon as the event is fired 
 deriving (Ord, Eq, Show)                

data Cond  = Obs String | -- a condition can be observable
             Nobs String  -- or non observable
 deriving (Ord, Eq, Show)                

-- Unsafe Action : this is the ouput of the algorithm : it is a
-- disjunction of conditional event: bad things may occur if we fire
-- some controllable event when some conjunction of observable
-- conditions hold:

data UnsafeAct = Any |			       -- unsafe
		 None |			       -- safe - NOTE: never used!
		 ContEv String |	       -- firing a controllable event
                 When UnsafeAct Condition |    -- only if the condition holds
     		 OrAct [UnsafeAct]             -- NEW! several cases
 deriving (Ord, Eq, Show)                

data Condition = TrueCond |              -- == AndCond [] 
		 FalseCond |
		 ObsVal String |	 -- reading an observable condition
		 AndCond [Condition] |   -- NEW! conjunction of conditions
		 OrCond Condition Condition    -- disjunction of conditions
 deriving (Ord, Eq, Show)                

-- the action function computes the output

   -- nothing can prevent the Nil Event from firing :

action NilEv = Any

   -- firing the event when the condition is true is not safe :

action (EC (Cont ev) cond) = When (ContEv ev) (value cond)

   -- if the condition can become true, an uncontrollable event can fire

action (EC (NCont ev) cond) = set_act cond

-- NOTE: essentially, "value" forgets about events (we have already found the
-- controllable one. In contrast, "set_act" does not - in fact, it recalls
-- "action" through "set_act_simple".

-----------------------------
-- NOTE: in the case of Controllable event, conditions are evaluated by
-- "value", whereas in the case of Uncontrollable, they are by "set_act"
--
-- The value function says on on which conditions a condition can
-- become true

-- value NilCond     = TrueCond
-- value (Cand sc c) = AndCond (valsimple sc) (value c)  -- straightforward
-- NEW:
value cs = AndCond $ map valsimple cs

   -- either the value is true or the up event can be fired

valsimple (CE cs e) = OrCond (val cs) (enable_cond e)
-- NOTE: valsimple evaluates SimpleCond

val (Obs cs)  = ObsVal cs
val (Nobs cs) = TrueCond

-- The enable_cond function says on which conditions a firing is enabled

-- enable_cond NilEv                = TrueCond
enable_cond (EC (Cont ev) cond)  = FalseCond 
enable_cond (EC (NCont ev) cond) = value cond 
-- NOTE: this one does the pruning. It's a pruned version of "value"

------------------------------
-- The set action says says which actions can set a condition
-- NEW
set_act :: [SimpleCond] -> UnsafeAct
set_act ls = case ls of 
   [] -> Any  -- NOTE: maybe "error" would be ok, as well.
   _ -> OrAct [When (set_act_simple x) (value $ delete x ls) | x <- ls]
                                              
-- set_act NilCond = Any
			
-- set_act (Cand cs c) = OrAct (When (set_act_simple cs) (value c))
                      -- either cs is set when c is true 
--                            (When (set_act c) (valsimple cs))
                      -- or c is set when cs is true

set_act_simple (CE cond ev) = action ev -- firing ev sets the condition
-- NOTE: compare with valsimple: here the condition is not considered, 
-- as it's not observable. It might be "FF OR (action ev)", but this 
-- conflicts with "val". 


