-- Paolo Torrini, Verimag, 2007

module VVlib where

import Prelude
import IO
import Debug.Trace
import List

trim :: String -> String
trim xs = (trimS . trimP . trimN . trimP) xs

trimP :: String -> String
trimP xs = case xs of  
  ' ':ys  -> trimP ys
  '\n':ys -> trimP ys
  _       -> xs

trimS :: String -> String
trimS xs = let 
    ys = trimP $ reverse xs
  in reverse ys

trimN :: String -> String
trimN xs = case xs of
  []     -> []
  '\n':_ -> []
  x:xs   -> x:(trimN xs)

----------------------- miscellaneous

numList :: [a] -> [(a,Int)]
numList ls = let 
     n = length ls 
   in nList n ls 
 where 
   nList n ys = case ys of
     [] -> []
     x:xs -> (x, n - length ys) : nList n xs

zipPr :: [([a],[b])] -> ([a],[b])
zipPr ls = case ls of
  [] -> ([],[])
  (bs,cs):xs -> let (ms,ns) = zipPr xs
     in (bs ++ ms, cs ++ ns)

mapJ :: (a -> Maybe b) -> [a] -> Maybe [b]
mapJ f xs = case xs of
  [] -> Just []
  y:ys -> do 
     zs <- mapJ f ys
     return $ (case f y of
        Just w -> w:zs
        _ -> zs)

negP :: (a -> Bool) -> (a -> Bool)
negP f = \x -> not $ f x

eqRR :: Eq a => [a] -> [a] -> Bool
eqRR h k = (genAnd (\x -> elem x h) k) && (genAnd (\x -> elem x k) h)

interNE :: Eq a => [a] -> [a] -> Bool
interNE bs cs = case (bs,cs) of
  ([],_)   -> False
  (_,[])   -> False
  (b:xs,_) -> if elem b cs then True else interNE xs cs
  
cProd :: Eq a => [[a]] -> [[a]]
cProd lss = case (delete [] lss) of
  [] -> [[]]
  xs:xss -> 
      [x:ws | x <- xs, ws <- cProd xss] 

----------------------------- and-or functions

genAnd :: (a -> Bool) -> [a] -> Bool
genAnd f ys = case ys of
               [] -> True
               x:xs -> if not (f x) then False
                           else (genAnd f xs)

genOr :: (a -> Bool) -> [a] -> Bool
genOr f ys = case ys of
               [] -> False
               x:xs -> if (f x) then True
                           else (genOr f xs)

genMAnd :: (a -> Maybe Bool) -> [a] -> Bool
genMAnd f ys = case ys of
               [] -> True
               x:xs -> case (f x) of
                  Just True -> genMAnd f xs
                  _ -> False

genMOr :: (a -> Maybe Bool) -> [a] -> Bool
genMOr f ys = case ys of
               [] -> False
               x:xs -> case (f x) of
                  Just True -> True
                  _ -> (genMOr f xs)

funAnd :: [a -> Bool] -> a -> Bool
funAnd fs x = case fs of
               [] -> True
               g:gs -> if not (g x) then False
                           else (funAnd gs x)

funOr :: [a -> Bool] -> a -> Bool
funOr fs x = case fs of
               [] -> False
               g:gs -> if (g x) then True
                           else (funOr gs x)

------------------------- printing functions

printList :: Show a => [a] -> IO [()]
printList ls = mapM (IO.hPrint IO.stdout) ls

printOut :: Show a => a -> IO ()
printOut c = do IO.hPrint IO.stdout c

printStrLn :: String -> IO ()
printStrLn c = do IO.hPutStrLn IO.stdout c

--------------- from Hets ----------------------------------------------

{-
-- remove duplicates from list
nub                     :: Eq a => [a] -> [a]
nub                     =  nubBy (==)

nubBy                   :: (a -> a -> Bool) -> [a] -> [a]
nubBy eq []             =  []
nubBy eq (x:xs)         =  x : nubBy eq (filter (\y -> not (eq x y)) xs)
-}